/* ACG.H -- Assembly Constraint Graph Header Module */
/* (c) 1988 - University of Michigan */

#ifndef _H_ACG
#define _H_ACG

#ifndef _H_CONFIG
#include "config.h"
#endif _H_CONFIG

#ifndef _H_SET
#include "set.h"
#endif _H_SET

#define DATA ACG_MOD

/* Assembly Graph
 * The sequence graph is the basis of the graph of sequence assertions.  This
 * is a fairly normal digraph.  The structure just stores the set of inpointing
 * and outpointing edges for each node.  For algorithmic reasons, we consider
 * each part both its own predecessor and its own successor.  This also
 * contains the set of trajectories which are possible for this part.
 */

typedef struct {
	PartSet pred;	 /* Set of parts preceeding this part */
	PartSet succ;	 /* Set of parts following this part */
	TrajSet traj;	 /* Possible trajectories for this part */
	setsize	npred;	 /* Number of predecessors */
	setsize	nsucc;	 /* Number of successors */
	setsize	ntraj;	 /* Number of trajectories */
#ifdef PRIVCON
	struct constra *cons;
#endif PRIVCON
	} Graf;

#ifdef NORMAL
struct norm {
	struct nc *next;
	PartSet p;
	setsize np;
	};
#endif NORMAL

/* Leaf Data for ACG Module
 * This contains the data structures used for a given node in the search tree.
 */

struct Leaf_Data {
	Graf *graf;	   /* Pointer to array of parts */
	int  traj_inc;	   /* # of parts with trajectory undetermined */
	int  sequ_inc;	   /* # of parts whose place in order is undetermined */
#ifdef NORMAL
	struct nc *norm; /* classes of similar parts in normal order */
#endif NORMAL
	};

/* Constraint List
 * This is a structure for keeping a linked list of constraints.  There is
 * an array pointers indexed by the part number, each of which points to a
 * linked list of structures like the one below.  These each give a set of
 * possible trajectories for the part, and a part that may not preceed the
 * part if any of those trajectories are used.  We can speed things up a
 * bit by giving each frame it's own copy of the constraint structure and
 * deleting constraints that have been satisfied, at the cost of more
 * memory and trouble.  This is done when PRIVCON is defined.
 */

typedef struct constra Constraint;

struct constra {
	TrajSet traj;	/* If part uses one of these trajectories */
	Part part;	/* Then this part better not be inserted first */
	Constraint *next;
	};

#ifdef PRIVCON
extern char *constraint_mm;
#else
extern Constraint **constraint_list;
#endif PRIVCON

/* Constraint Macros */

/* Pointer to the first constraint for a given part in a given graf */
#ifdef PRIVCON
#define constraint1(g,p) g[p].cons
#define drop_constraint(c) mm_free(constraint_mm,c)
#define alloc_constraint() (Constraint *) mm_alloc(constraint_mm)
#define init_priv_con() constraint_mm = mm_init(sizeof(Constraint),40)
#else
#define constraint1(g,p) constraint_list[p]
#define drop_constraint(c) m_free(c,sizeof(Constraint))
#define alloc_constraint() (Constraint *) m_alloc(sizeof(Constraint))
#endif PRIVCON

#endif _H_ACG
