/* ACG_DUMP.C -- Routines to print acg data structures */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "config.h"
#include "global.h"
#include "set.h"
#include "acg.h"

/* DUMP_CONSTRAINTS(fp)
 * Output the constraint data structure to fp.
 */

dump_constraints(leaf,fp)
Leaf leaf;
FILE *fp;
{
int parts;
Part p;
Traj t;
Constraint *c;

	parts = part_number();

	for (p = 0; p < parts; p++)
	{
		for (c=constraint1(leaf[DATA]->graf,p); c != NULL; c = c->next)
		{
			fprintf(fp,"%s->%s if trajectory is ",
				part_name(p),part_name(c->part));
			pr_traj_or(fp,&c->traj);
			fprintf(fp,"\n");
		}
	}
}

/* ACG_DUMP(leaf,fp)
 * Output the data structures in the given leaf to fp.
 */

acg_dump(leaf,fp)
Leaf leaf;
FILE *fp;
{
int parts = part_number();
Graf *graf = leaf[DATA]->graf;
Part p;
Traj t;

	for (p = 0; p < parts; p++)
	{
		fprintf(fp,"part: %s\n",part_name(p));
		fprintf(fp,"  preceeded by: ");
		pr_part_and(fp,&graf[p].pred);
		fprintf(fp," (%d)\n  succeeded by: ",graf[p].npred);
		pr_part_and(fp,&graf[p].succ);
		fprintf(fp," (%d)\n  trajectories: ",graf[p].nsucc);
		pr_traj_or(fp,&graf[p].traj);
		fprintf(fp," (%d)\n",graf[p].ntraj);
	}
#ifdef PRIVCON
	dump_constraints(leaf,fp);
#endif PRIVCON
	fprintf(fp,"incomplete: %d traj %d sequ\n",
		leaf[DATA]->traj_inc, leaf[DATA]->sequ_inc);
}

/* ACG_PRINT(leaf,fp)
 * Output a complete plan to fp.  It is an error to call this with an
 * incomplete plan.
 */

acg_print(leaf,fp)
Leaf leaf;
FILE *fp;
{
int parts = part_number();
Graf *graf = leaf[DATA]->graf;
int n;
Part p;

	if (leaf[DATA]->sequ_inc > 0 || leaf[DATA]->traj_inc > 0)
		error("attempt to print an incomplete plan\n");

	for (n=1; n <= parts; n++)
		for (p=0; p < parts; p++)
			if (graf[p].npred == n)
			{
				fprintf(fp,"(%d) %s using ",n,part_name(p));
				pr_traj_or(fp,&graf[p].traj);
				fprintf(fp,"\n");
				break;
			}
}
