/* ACG_CRIT.C -- Routines to handle the default criteria */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "config.h"
#include "global.h"
#include "set.h"
#include "acg.h"
#include "assert.h"

short nconstraint[MAXPARTS][(MAXPARTS>MAXTRAJS)?MAXPARTS:MAXTRAJS];
short nassert[MAXPARTS];

int dflt_prop(leaf,prop)
Leaf leaf;
Prop *prop;
{
register Part p,q;
register Traj t;
register Constraint *c;
register int n;
Graf *graf = leaf[DATA]->graf;
int parts = part_number();
int trajs = traj_number();
static SequProp sp;
static TrajProp tp;
short best_sequ, best_traj;

	if (is_on(SHOW_ENTRIES)) printf("dflt_prop\n");

	/* SEQUENCING CONSTRAINTS */

	/* Count up number of assertions we are involved in */
	for (p=0;p<parts;p++)
		if ((n = graf[p].npred + graf[p].nsucc) != parts + 1)
			nassert[p] = n - 2;
		else
			nassert[p] = -1;

	for (p=1;p<parts;p++)
	    if (nassert[p] >= 0)
		for (q=0;q<p;q++)
		    if (nassert[q] < 0 ||
			isin(&graf[p].pred,q) || isin(&graf[p].succ,q))
			    nconstraint[q][p] = -1;
		    else
			    nconstraint[q][p] = nassert[p] + nassert[q];

	/* Count up unactivated sequencing constraints */
	for (p=0;p<parts;p++)
	    if (nassert[p] >= 0)
		for (c=constraint1(graf,p); c != NULL;  c=c->next)
#ifndef PRIVCON
			/* If global constraint list, test if unactivated */
			if (!isin(&graf[p].pred,c->part) &&
			    !isin(&graf[p].succ,c->part))
#endif PRIVCON
			{
				if (p < c->part)
					nconstraint[p][c->part]++;
				else
					nconstraint[c->part][p]++;
			}

	/* Look for best pair */
	best_sequ = -1;
	for (p=1;p<parts;p++)
	    if (nassert[p] >= 0)
		for (q=0;q<p;q++)
		    if (nconstraint[q][p] > best_sequ)
		    {
			sp.p1 = p;
			sp.p2 = q;
			best_sequ = nconstraint[q][p];
		    }

	/* TRAJECTORY CONSTRAINTS */

	for (p=0;p<parts;p++)
		for (t=0;t<trajs;t++)
			nconstraint[p][t] = 0;

	/* Count unfired consequences of single trajectories and negations */
	for (p=0;p<parts;p++)
	    if (graf[p].ntraj > 1)
	    {
		setcopy(&tp.t,&graf[p].traj);
		foreach(t,&tp.t)
		{
			remove1(&tp.t,t);
			for (c=constraint1(graf,p); c != NULL;  c=c->next)
#ifdef PRIVCON
			    if ((isin(&c->traj,t) || issubset(&tp.t,&c->traj)))
#else
			    if (!isin(&graf[p].pred,c->part) &&
				!isin(&graf[p].succ,c->part) &&
				(isin(&c->traj,t) || issubset(&tp.t,&c->traj)))
#endif PRIVCON
				    nconstraint[p][t]++;
			append1(&tp.t,t);
		}
	    }

	/* Pick best trajectory assertion */
	best_traj = -1;
	for (p=0;p<parts;p++)
	    if (graf[p].ntraj > 1)
		for (t=0;t<trajs;t++)
		    if (nconstraint[p][t] > best_traj
			&& nconstraint[p][t] > best_sequ
			&& (nconstraint[p][t] > 0 || isin(&graf[p].traj,t)))
		    {
			    tp.p = p;
			    settrajset(&tp.t,t);
			    best_traj = nconstraint[p][t];
		    }

	/* Pick best assertion */
	if (best_traj > best_sequ)
	{
		prop->type = TRAJ;
		prop->a.traj = &tp;
	}
	else
	{
		prop->type = SEQU;
		prop->a.sequ = &sp;
	}
	return(0);
}

int dflt_test(leaf,prop)
Leaf leaf;
Prop *prop;
{
Graf *graf = leaf[DATA]->graf;
register Part p,q;
register int n;
TrajSet *ts1,*ts2,tsb;

register Constraint *c;
int parts = part_number();
int trajs = traj_number();
static SequProp sp;
static TrajProp tp;
short best_sequ, best_traj;

    if (is_on(SHOW_ENTRIES)) printf("dflt_test\n");

    if (prop->type == SEQU)
    {
	/* SEQUENCING CONSTRAINTS */
	p = prop->a.sequ->p1;
	q = prop->a.sequ->p2;

	/* Count up number of assertions we are involved in */
	n = graf[q].npred + graf[q].nsucc + graf[p].npred + graf[p].nsucc;

	/* Count up unactivated sequencing constraints */
	for (c=constraint1(graf,p); c != NULL;  c=c->next)
	   if (c->part == q)
		n++;

	for (c=constraint1(graf,q); c != NULL;  c=c->next)
	   if (c->part == p)
		n++;

	return (nconstraint[q][p]);
    }
    else
    {
	/* TRAJECTORY CONSTRAINTS */
	p = prop->a.traj->p;
	setdifference((ts2 = &tsb),&graf[p].traj,(ts1 = &prop->a.traj->t));

	n = 0;
	for (c=constraint1(graf,p); c != NULL;  c=c->next)
#ifdef PRIVCON
	    if ((issubset(ts1,&c->traj) || issubset(ts2,&c->traj)))
#else
	    if (!isin(&graf[p].pred,c->part) &&
		!isin(&graf[p].succ,c->part) &&
	        (issubset(ts1,&c->traj) || issubset(ts2,&c->traj)))
#endif PRIVCON
		    n++;
	    }

	return(n);
}

/* Do nothing entries here just for consistancy */
dflt_init() { }
dflt_copy() { }
dflt_drop() { }
dflt_sequ() { }
dflt_traj() { }
dflt_dump() { }
int dflt_rate() { return(0); }

#ifdef SIZE
long dflt_stat_size() {return(0);}

dflt_leaf_size(leaf,size,links)
Leaf leaf;
long *size;
int *links;
{
	*size = 0;
	*links = 1;
}
#endif SIZE
