/* ACG_SIM.C -- Routines to identify similar plans */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "config.h"
#include "global.h"
#include "set.h"
#include "acg.h"
#include "assert.h"

#ifndef PRIVCON
ERROR PRIVCON must be defined when using similarity checking
#endif

/* Temorary data structure used to build normalized set classes */
struct normap {
	Part part;
	char built;
	short tcc[MAXTRAJS];
	};

Leaf cleaf;
int cmp_parts();

/* MAKE_NORM(leaf)
 * This rebuilds the normal set class list for the given leaf.  Before calling
 * this, leaf->norm should be either NULL or point to a list of sets that is
 * sane enought to deallocate.
 */

make_norm(leaf)
Leaf leaf;
{
struct normap nm[MAXPARTS];
register Part p;
struct nc *n, *nt;

	for (p = 0; p < part_number(); p++)
	{
		nm[p]->built = FALSE;
		nm[p]->part = p;
	}

	cleaf = leaf;
	qsort(nm,part_number(),sizeof(struct normap),cmp_parts);

	if (leaf[DATA]->norm != NULL) drop_norm(leaf);
	n = leaf[DATA]->norm = (struct nc *) m_alloc(sizeof(struct nc));
	setpartset(&n->p,nm[p]->part);

	for (p = 1; p < part_number(); p++)
	{
		if (cmp_parts(&nm[p],&nm[p-1]) == 0)
			append1(&n->p,nm[p]->part);
		else
		{
			nt = (struct nc *) m_alloc(sizeof(struct nc));
			setpartset(&n->p,nm[p]->part);
			n->next = nt;
			n->np = cardinality(&n->p);
			n = nt;
		}
	}
	n->next = NULL;
}


drop_norm(leaf)
Leaf leaf;
{
struct nc *n1,*n2;

	for (n1=leaf[DATA]->norm; n1 != NULL; n1=n2)
	{
		n2 = n1->next;
		m_free(n1,sizeof(struct nc));
	}
}


/* CMP_PARTS(p1,p2)
 * This is part of the normalization process.  It returns:
 *   -1   if p1 preceeds p2 in the normalization.
 *    0   if p1 and p2 are superficially similar.
 *    1   if p2 preceeds p1 in the normalization.
 * The global cleaf tells what leaf we are working on.
 */

int cmp_parts(pp1,pp2)
struct normap *pp1, *pp2;
{
register Part p1=pp1->part,p2=pp2->part;
Graf *graf = cleaf[DATA]->graf;
int c1[MAXTRAJS], c2[MAXTRAJS];
register Traj t;

	if (graf[p1].ntraj != graf[p2].ntraj)
		return((graf[p1].ntraj < graf[p2].ntraj): -1 : 1);
	if (graf[p1].npred != graf[p2].npred)
		return((graf[p1].npred < graf[p2].npred): -1 : 1);
	if (graf[p1].nsucc != graf[p2].nsucc)
		return((graf[p1].nsucc < graf[p2].nsucc): -1 : 1);

	mk_cc(pp1);
	mk_cc(pp2);
	for (t = 0; t < traj_number(); t++)
		if (cc[p1][t] != cc[p2][t])
		    return((cc[p1][t] < cc[p2][t] -1 : 1);
}


/* MK_CC(p)
 * This builds a list of constraint counts for the named part in leaf cleaf.
 */

int icp (a,b) int *a,*b; { return(*a-*b); }

mk_cc(p)
struct normap *pp;
{
Graf *gp = &cleaf[DATA]->graf[p];
Part p = pp->part;
register Constraint *c;
register Traj t;

	if (pp->built) return;
	pp->built = TRUE;
	for (t = 0; t < traj_number(); t++)
		pp->tcc[t] = 0;

	for (c = constraint_list[p]; c != NULL; c = c->next)
	    /* For constraints whose conclusions are not already asserted */
	    if (!isin(&gp->succ,c->part))
		/* Add one to count of each trajectory that is still possible */
		foreach(t,&c->traj)
		    if (isin(&gp->traj,t))
			pp->tcc[t]++;

	qsort(pp->tcc,traj_number(),sizeof(int),icp);
}

int mkclist(leaf,cons)
Leaf leaf;
Constraint **cons;
{
Graf *graf;
Part p;
Constraint *c;

	foreachpart(p)
	{
		graf = leaf[DATA]->graf[p];
		for (c=constraint_list[p]; c != NULL; c = c->next)
		{
			if (!isin(&graf->succ,c->part) &&
			    !isin(&graf->pred,c->part) &&
			    !disjoint(&graf->traj,&c->traj))
			{
				
			}
		}
	}

sim_map(leaf1,leaf2)
Leaf leaf1, leaf2;
{
struct nc *n1, *n2;
Constraint *clist1[MAXPARTS], *clist2[MAXPARTS];
Graf *g1;
Graf *g2;

	/* First, see if the number and cardinality of normal classes is the
	   same for both */
	
	n1 = leaf1[DATA]->norm;
	n2 = leaf1[DATA]->norm;
	for(;;)
	{
		if (n1 == NULL)
		{
			if (n2 == NULL)
				break;
			else
				return (NULL);
		}
		else
		{
			if (n2 == NULL)
				return(NULL);
			else
				if (n1->np != n2->np) return(NULL);
		}
		n1 = n1->next;
		n2 = n2->next;
	}

	/* Next, check if the classes have the same cardinalities */

	for(n1 = leaf1[DATA]->norm,n2 = leaf1[DATA]->norm;
	    n1 != NULL; n1 = n1->next, n2 = n2->next)
	{
		g1 = leaf1[DATA]->graf[first_in(n1->p)];
		g2 = leaf2[DATA]->graf[first_in(n2->p)];
		if (g1->npred != g2->npred ||
		    g1->nsucc != g2->nsucc ||
		    g1->ntraj != g2->ntraj) return(NULL);
	}

	/* Now count up the unsatisfied constraints for each */
	mkclist(leaf1,clist1);
	mkclist(leaf1,clist1);
-----

	/* Find the smallest part class */

	/* Pick a part from part class in plan 1 */
	foreach (p1, smallclass)
	{
		/* Pick a part from part class in plan 2 */
		for each p2 in corrisponding class, in order of similarness
		{
			/* Propose that part mapping */
			if (pro_map(p1,p2))
				goto good;
		}
		/* Can't map p1 to anything */
		return(BAD);
good:		;
	}

boolean pro_pmap(p1,p2)
{
	pmap[p1] = p2;

	/* Check if successors map */
	unk = 0;
	if (graf1[p1].nsucc > 0)
	    foreach (p3,&graf1[p1].succ)
		if (p3 != p1)
		{
			if (pmap[p3] < 0)
			{
			    if (!is_in(&graf2[p2].succ,pmap[p3]))
				return (FALSE);
			}
			else
			{
			    unk++;
			    unk_p1 = p3;
			}
		}

	/* If only one unmapped successor, map it */
	if (unk == 1)
	{
		foreach (p3, &graf2[p2].succ)
			if (pmap[p3] < 0)
			{
				if (!pro_map(unk_p1,p3)) return(FALSE);
				break;
			}
	}

	/* Check if predecessors map */
	unk = 0;
	if (graf1[p1].npred > 0)
	    foreach (p3,&graf1[p1].pred)
		if (p3 != p1)
		{
			if (pmap[p3] < 0)
			{
			    if (!is_in(&graf2[p2].pred,pmap[p3]))
				return (FALSE);
			}
			else
			{
			    unk++;
			    unk_p1 = p3;
			}
		}

	/* If only one unmapped predecessor, map it */
	if (unk == 1)
	{
		foreach (p3, &graf2[p2].pred)
			if (pmap[p3] < 0)
			{
				if (!pro_map(unk_p1,p3)) return(FALSE);
				break;
			}
	}

	/* Check if trajectories map */
