/* ACG_UTIL.C -- Routines to fuddel various things with proposals */
/* (c) 1988 - University of Michigan */

#include "config.h"
#include "global.h"
#include "set.h"
#include "acg.h"
#include "assert.h"

/* PROP_COPY(p1,p)
 * This sets p1 to a copy of p.  The copy is kept in local static storage,
 * so a second call to this will obliterate the results of the previous call.
 */

prop_copy(p1,p)
Prop *p1, *p;
{
static SequProp sp1;
static TrajProp tp1;
	
	p1->type = p->type;

	if (p->type == TRAJ)
	{
		p1->a.traj = &tp1;
		tp1.p = p->a.traj->p;
		setcopy(&tp1.t,&p->a.traj->t);
	}
	else
	{
		p1->a.sequ = &sp1;
		sp1.p1 = p->a.sequ->p1;
		sp1.p2 = p->a.sequ->p2;
	}
}

/* PROP_NEGATE(leaf,p2,p)
 * This sets p2 to the negation of p within the given leaf.  The copy is kept
 * in local static storage, so a second call to this will obliterate the
 * results of the previous call.
 */

prop_negate(leaf,p2,p)
Leaf leaf;
Prop *p2, *p;
{
static SequProp sp2;
static TrajProp tp2;
	
	p2->type = p->type;

	if (p->type == TRAJ)
	{
		p2->a.traj = &tp2;
		tp2.p = p->a.traj->p;
		setdifference(&tp2.t,
			&leaf[DATA]->graf[tp2.p].traj,
			&p->a.traj->t);
	}
	else
	{
		p2->a.sequ = &sp2;
		sp2.p2 = p->a.sequ->p1;
		sp2.p1 = p->a.sequ->p2;
	}
}

/* PROP_EQUAL(leaf,p1,p2)
 * This tests if two proposals are equal.  This is true if they are either
 * the same or if one is a negation of the other.
 */
boolean prop_equal(leaf,p1,p2)
Leaf leaf;
Prop *p1,*p2;
{
register SequProp *sp1,*sp2;
register TrajProp *tp1,*tp2;
TrajSet ts;

	if (p1->type != p2->type)
		return(FALSE);
	
	if (p1->type == SEQU)
		return (((p1->a.sequ->p1 == p2->a.sequ->p1) &&
		        (p1->a.sequ->p2 == p2->a.sequ->p2)) ||
		       ((p1->a.sequ->p1 == p2->a.sequ->p2) &&
		        (p1->a.sequ->p2 == p2->a.sequ->p1)));
	else
	{
		if (p1->a.traj->p != p2->a.traj->p)
			return(FALSE);
		if (isequal(&p1->a.traj->t,&p2->a.traj->t))
			return(TRUE);
		setdifference(&ts,
		     &leaf[DATA]->graf[p1->a.traj->p].traj,
		     &p1->a.traj->t);
		return (isequal(&ts,&p2->a.traj->t));
	}
}

/* POSSIBLE_TRAJ(leaf,p)
 * Return a pointer to the current set of possible trajectories for part p
 */

TrajSet *possible_traj(leaf,p)
Leaf leaf;
Part p;
{
	return(&leaf[DATA]->graf[p].traj);
}

/* N_PRED_PARTS(leaf,p)
 * Return the number of preceeding parts for part p
 */

int n_pred_parts(leaf,p)
Leaf leaf;
Part p;
{
	return(leaf[DATA]->graf[p].npred-1);
}

/* IS_SUCC(leaf,p,q)
 * Return TRUE if q is a successor of p.
 */

boolean is_succ(leaf,p,q)
Leaf leaf;
Part p,q;
{
	return(isin(&leaf[DATA]->graf[p].succ,q));
}


#ifdef DISCARD
/* REDUCE_PROP(leaf,prop)
 * This converts prop into a contained legal proposal if possible,
 * and returns TRUE if the proposal produced is redundant.
 */

boolean reduce_prop(leaf,prop)
Leaf leaf;
Prop *prop;
{

	if (prop->type == SEQU)
		/* Test if we already have it.  Return TRUE if we do. */
		return(isin(&leaf[DATA]->graf[prop->a.sequ->p2].pred,
			    prop->a.sequ->p1));
	else
		/* Intersect proposed trajectories with possible trajectories
		   Returning TRUE if the result is empty. */
		if (!intersect(&prop->a.traj->t,
			      &leaf[DATA]->graf[prop->a.traj->p].traj))
			return(TRUE);
		return(isequal(&prop->a.traj->t,
			       &leaf[DATA]->graf[prop->a.traj->p].traj));
}
#endif DISCARD
