/* CAD_DUMP.C -- Routines to print out CAD Interface Data Structures */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "config.h"
#include "set.h"
#include "global.h"
#include "cad.h"

dump_names(fp)
FILE *fp;
{
Part p;
Traj t;
Dire d;

	fprintf(fp,"%d parts:\n",n_parts);
	for (p=0;p<n_parts;p++) fprintf(fp,"   %d: %s\n",p,p_name[p]);

	fprintf(fp,"%d trajectories:\n",n_trajectories);
	for (t=0;t<n_trajectories;t++) fprintf(fp,"   %d: %s\n",t,t_name[t]);

	fprintf(fp,"%d directions:\n",n_directions);
	for (d=0;d<n_directions;d++) fprintf(fp,"   %d: %s\n",d,d_name[d]);
}

dump_prop_traj(fp)
FILE *fp;
{
Part p;

	for (p=0;p<n_parts;p++)
	{
		fprintf(fp,"part %s trajs: ",p_name[p]);
		pr_traj_or(fp,proposed[p]);
		fprintf(fp,"\n");
	}
}

dump_crit(fp)
FILE *fp;
{
Part p;
struct con *c;

	for (p=0;p<n_parts;p++)
		for (c=constraint[p]; c!=NULL; c=c->next)
		{
			fprintf(fp,"if %s uses %s then %s -> ",
				p_name[p],t_name[c->tr], p_name[p]);
			pr_part_and(fp,c->sp);
			fprintf(fp,"\n");
		}
}
	
dump_dsfc(fp)
FILE *fp;
{
Part p;
int i;
struct rset *rsp;

	for (p=0;p<n_parts;p++)
	{
		fprintf(fp,"part %s attach: ",p_name[p]);
		pr_part_or(fp,attach[p]);
		fprintf(fp,"\npart %s lift: ",p_name[p]);
		pr_part_or(fp,lift[p]);
		for (rsp = rotate[p], i=1; rsp != NULL; rsp=rsp->next, i++)
		{
			fprintf(fp,"\npart %s rotate %d: ",p_name[p],i);
			pr_part_or(fp,rsp->p);
		}
		fprintf(fp,"\n");
	}
}

dump_dmanip(fp)
FILE *fp;
{
Part p;
Traj t;

	for (p=0;p<n_parts;p++)
		fprintf(fp,"part %s manip: %d\n",p_name[p],manip[p]);
	for (t=0;t<n_trajectories;t++)
		fprintf(fp,"traj %s diffic: %d\n",t_name[t],diffic[t]);
}

dump_dmono(fp)
FILE *fp;
{
Dire d;
	for (d=0;d<n_directions;d++)
	{
		fprintf(fp,"direction %s trajectories:  ",d_name[d]);
		pr_traj_and(fp,dir_traj[d]);
		fprintf(fp,"\n");
	}
}

#ifdef SIZE
long cad_size()
{
register Part p;
register Dire d;
struct con *c;
struct rset *rsp;
long size = 0;
long cad_weight_size();
long cad_switch_size();

	for (p=0; p<n_parts; p++)
	{
 	   if (constraint[p] != NULL)
	   {
		for (c=constraint[p]; c != NULL; c = c->next)
			size += sizeof(struct con) + setmem(c->sp);
 
		size += setmem(proposed[p]) +
		        setmem(attach[p]) +
		        setmem(lift[p]);
#ifdef CONNECT
		size += setmem(connect[p]);
#endif CONNECT
		for (rsp = rotate[p];rsp != NULL; rsp=rsp->next)
			size += sizeof(struct rset);
	    }
	}
	for (d=0; d<n_directions; d++)
	    if (dir_traj[d] != NULL)
	    {
		size += setmem(dir_traj[d]);
	    }
 
	size += cad_weight_size();
	size += cad_switch_size();
}
#endif SIZE
