/* CAD_QUERY.C -- Routines to query fake cad data base */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "config.h"
#include "set.h"
#include "cad.h"

/* CONSTRAINED_PARTS(p,t)
 * These routines return the set of parts which must be installed after p if
 * p is inserted with trajectory t.  If there are none, it may return either
 * an empty set or a NULL, depending on the state of the database.
 */

PartSet *constrained_parts(p,t)
Part p;
Traj t;
{
struct con *c1;

	/* Search the list of constraints for that one */
	for (c1 = constraint[p]; c1 != NULL; c1 = c1->next)
		if (c1->tr == t)
			return(c1->sp);

	return(NULL);
}

	
/* PROPOSED_TRAJ(p)
 * Return the set of trajectories that were proposed for the part p.  
 */

TrajSet *proposed_traj(p)
Part p;
{
	return(proposed[p]);
}

/* PART_NAME(p)
 * DIRE_NAME(d)
 * TRAJ_NAME(t)
 * Routines to return name of something.  This is your standard object-oriented
 * foolishness.
 */

char *part_name(p)
Part p;
{
	return(p_name[p]);
}

char *dire_name(d)
Dire d;
{
	return(d_name[d]);
}

char *traj_name(t)
Traj t;
{
	return(t_name[t]);
}

/* GET_N_AXIS(p)
 * Return the number of rotational axis for part p.
 */

int get_n_axis(p)
Part p;
{
struct rset *rsp;
int n = 0;

	for(rsp=rotate[p]; rsp != NULL; rsp=rsp->next)
		n++;
	return(n);
}


/* GET_ATTACHED(p)
 * GET_LIFT(p)
 * GET_ROTATE(p,a)
 * GET_CONTACT(p)
 * Return the set of the set of parts to which p is attached, lift-only, or
 * on which it rotates about axis a.
 */

PartSet *get_attached(p)
Part p;
{
	return(attach[p]);
}

PartSet *get_lift(p)
Part p;
{
	return(lift[p]);
}

PartSet *get_rotate(p,a)
Part p;
register int a;
{
register struct rset *rsp;

	rsp=rotate[p];
	for(;;)
	{
		if (rsp == NULL) return(NULL);
		if (a-- == 0) return(&rsp->p);
		rsp=rsp->next;
	}
}

#ifdef CONTACT
PartSet *get_contact(p)
Part p;
{
	return(contact[p]);
}
#endif CONTACT

/* GET_MANI(part)
 * GET_DIFF(traj)
 * Get the manipulability or difficulty scores for a part or trajectory.
 */

int get_mani(p)
Part p;
{
	return(manip[p]);
}

int get_diff(t)
Traj t;
{
	return(diffic[t]);
}

/* GET_DIR_TRAJ(dir)
 * Return the set of trajectories that uses the given direction.
 */
TrajSet *get_dir_traj(dir)
Dire dir;
{
	return(dir_traj[dir]);
}
