/* CRIT.C -- Criteria top-level enties */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "assert.h"
#include "global.h"

#define DATA CRIT_MOD

int sfc_init(), sfc_copy(), sfc_sequ(), sfc_traj();
int sfc_rate(), sfc_prop(), sfc_drop(), sfc_dump();
int sfc_test();
int mani_init(), mani_copy(), mani_sequ(), mani_traj();
int mani_rate(), mani_prop(), mani_drop(), mani_dump();
int mani_test();
int mono_init(), mono_copy(), mono_sequ(), mono_traj();
int mono_rate(), mono_prop(), mono_drop(), mono_dump();
int mono_test();
int dflt_init(), dflt_copy(), dflt_sequ(), dflt_traj();
int dflt_init(), dflt_copy(), dflt_sequ(), dflt_traj();
int dflt_rate(), dflt_prop(), dflt_drop(), dflt_dump();
int dflt_test();


/* List of Criteria
 * These are the various criteria.  This consists of the criteria's weight
 * and the addresses of its various routines.
 */

struct Crit_List {
	char *name;	/* Name of the criterion */
	int eweight;	/* Evaluation Weight assigned to the criterion */
	int pweight;	/* Proposal Weight assigned to the criterion */
	boolean on;	/* is the criterion active? */
	int (*init)();	/* Pointer to initialization routine */
	int (*copy)();	/* Pointer to leaf copy routine */
	int (*sequ)();	/* Pointer to sequence assertion update routine */
	int (*traj)();	/* Pointer to trajectory assertion updata routine */
	int (*rate)();	/* Pointer to rating routine */
	int (*prop)();	/* Pointer to proposal routine */
	int (*drop)();	/* Pointer to data structure discard routine */
	int (*dump)();	/* Pointer to data structure dump routine */
	int (*test)();	/* Pointer to proposal test entry point */
	};

#define N_CRITERIA	4

struct Crit_List crittab[N_CRITERIA] = {
	{ "SFC", 1, 1, TRUE,
	  sfc_init, sfc_copy, sfc_sequ, sfc_traj,
	  sfc_rate, sfc_prop, sfc_drop, sfc_dump, sfc_test
	},
	{ "MANIP", 1, 1, TRUE,
	  mani_init, mani_copy, mani_sequ, mani_traj,
	  mani_rate, mani_prop, mani_drop, mani_dump, mani_test
	},
	{ "MONOD", 1, 1, TRUE,
	  mono_init, mono_copy, mono_sequ, mono_traj,
	  mono_rate, mono_prop, mono_drop, mono_dump, mono_test
	},
	/* Default criterion must be the last listed */
#define DEFAULT_CRITERION N_CRITERIA-1
	{ "DEFAULT", 1, 0, TRUE,
	  dflt_init, dflt_copy, dflt_sequ, dflt_traj,
	  dflt_rate, dflt_prop, dflt_drop, dflt_dump, dflt_test
	} };

crit_init(leaf,fp)
Leaf leaf;
FILE *fp;
{
int i;
	/* Read in the weights of the criteria */
	/* Turn on ones with non-zero weights and the DEFAULT criterion */
	for (i=0; i < N_CRITERIA; i++)
	{
	    getweights(crittab[i].name,fp,
		    &crittab[i].eweight,&crittab[i].pweight);
	    if (i == DEFAULT_CRITERION)
		    crittab[i].on = TRUE;
	    else
		    crittab[i].on = crittab[i].eweight || crittab[i].pweight;
	    if (is_on(SHOW_WEIGHTS))
		    printf("%s weights = %d %d\n", crittab[i].name,
			    crittab[i].eweight,crittab[i].pweight);
	}

	/* Initialize every criterion */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
			(*crittab[i].init)(leaf);
}

crit_dump(leaf,fp)
Leaf leaf;
FILE *fp;
{
int i;
	/* Dump every criterion */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
			(*crittab[i].dump)(leaf,fp);
}

crit_copy(new,old)
Leaf new,old;
{
int i;
	/* Have every criterion do it's copy */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
			(*crittab[i].copy)(new,old);
}


crit_drop(leaf)
Leaf leaf;
{
int i;
	/* Have every criterion do it's drop */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
			(*crittab[i].drop)(leaf);
}

crit_sequ(leaf,sa)
Leaf leaf;
SequAsrt *sa;
{
int i;
	/* Have every criterion do it's sequencing criterion update */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
			(*crittab[i].sequ)(leaf,sa);
}

crit_traj(leaf,ta)
Leaf leaf;
TrajAsrt *ta;
{
int i;
	/* Have every criterion do it's trajectory criterion update */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
			(*crittab[i].traj)(leaf,ta);
}

int crit_rate(leaf)
Leaf leaf;
{
int i,rating=0,this_rate;

	/* Have every real criterion do it's rating */
	for (i=0; i < N_CRITERIA-1; i++)
		if (crittab[i].eweight)
		{
			this_rate=(*crittab[i].rate)(leaf);
			rating += crittab[i].eweight*this_rate;
			if (is_on(SHOW_RATINGS)) printf("%s = %d  ",
				crittab[i].name, this_rate);
		}
	if (is_on(SHOW_RATINGS)) printf("\n");
	return(rating);
}

#ifndef TEST_PROPOSALS

crit_prop(leaf,prop1,prop2)
Leaf leaf;
Prop *prop1, *prop2;
{
int i,j,k;
Prop p[N_CRITERIA-1];
int r[N_CRITERIA-1];

    /* Have every real criterion do it's proposal */
    for (i=0; i < N_CRITERIA-1; i++)
	if (j=crittab[i].pweight)
	{
	    r[i] = j*(*crittab[i].prop)(leaf,&p[i]);
	    if (is_on(SHOW_PROPS))
	    {
		if (r[i] >= 0)
		{
			printf("%s weight %d proposes ",crittab[i].name,r[i]);
			dump_prop(stdout,&p[i]);
		}
		else
			printf("%s passes\n",crittab[i].name);
	    }
	    if (r[i] >= 0)
		for (k=0; k < i; k++)
		   if (crittab[k].pweight && r[k] >= 0 &&
		       prop_equal(leaf,&p[i],&p[k]))
		   {
			r[k] += r[i];
			r[i] = -1;
			break;
		   }
	}
	else
		r[i] = -1;

    /* Pick the highest rated proposal */
    j = -1;
    for (i=0; i < N_CRITERIA-1; i++)
	if (r[i] > j ||
	    (j == 0 && r[i] == 0 && crittab[i].pweight > crittab[k].pweight))
		j = r[k=i];

    /* If nothing proposed, use dflt criterion (always the last one) */
    if (j == -1)
    {
	(*crittab[DEFAULT_CRITERION].prop)(leaf,&p[k=0]);
	if (is_on(SHOW_TREE))
		printf("proposal from %s\n",crittab[DEFAULT_CRITERION].name);
    }
    else
	if (is_on(SHOW_TREE))
		printf("proposal from %s\n",crittab[k].name);

    /* Negate the proposal */
    prop_copy(prop1,&p[k]);
    prop_negate(leaf,prop2,&p[k]);
}

#else

crit_prop(leaf,prop1,prop2)
Leaf leaf;
Prop *prop1, *prop2;
{
register int i,j;
int k;
int lastprop;
int cnt;
int weight;
int bestprop, bestrating;
Prop p[N_CRITERIA];		/* Proposals from each criterion */
int pr[N_CRITERIA];		/* Ratings from proposer for p[i] */
int total;

    /* Have every real criterion do it's proposal */
    cnt = 0;
    for (i=0; i < N_CRITERIA-1; i++)
	if (weight=crittab[i].pweight)
	{
	    if ((pr[i] = weight*(*crittab[i].prop)(leaf,&p[i])) >= 0)
	    {
		lastprop = i;
		cnt++;
	    }
	}
	else
		pr[i] = -1;

    if (cnt == 0)
    {
        /* If nothing proposed, use dflt criterion (always the last one) */
	if (is_on(SHOW_PROPS))
		printf("no proposals\n");
	bestprop = DEFAULT_CRITERION;
	bestrating = pr[bestprop] =
		(*crittab[bestprop].prop)(leaf,&p[bestprop]);
    }
    else if (cnt == 1)
    {
	/* If only one proposal, use it */
	if (is_on(SHOW_PROPS))
	    printf("only proposal is from %s weight %d\n",
		crittab[lastprop].name,pr[lastprop]);
	bestprop = lastprop;
	bestrating = pr[lastprop];
    }
    else
    {
    	/* Let criteria test each other's proposals, add em, pick highest */
	bestrating = -1;
	for (i=0; i <= lastprop; i++)
	    if (pr[i] >= 0)
	    {
		if (is_on(SHOW_PROPS))
		    printf("proposal from %s (%d)",crittab[i].name,pr[i]);

		/* Get other criteria's opinions, and sum them up */
		total = 0;
		for (j=0; j < N_CRITERIA; j++)
		{
		    if (j == i)
			total += pr[i];
		    /* Use only criteria that proposed things and DEFAULT */
		    else if (j == DEFAULT_CRITERION || pr[j] > 0)
		    {
			total += k = crittab[j].pweight *
			             abs((*crittab[j].test)(leaf,&p[i]));
			if (is_on(SHOW_PROPS) && k > 0)
			    printf(" + %s (%d)",crittab[j].name,k);
		    }
		}

		if (is_on(SHOW_PROPS))
		{
		    printf(" weight %d: ",total);
		    dump_prop(stdout,&p[i]);
		}

		/* Look for best total rating */
		if (total > bestrating ||
		    (bestrating == 0 && total == 0 &&
		    crittab[i].pweight > crittab[j].pweight))
		{
			bestrating = total;
			bestprop=i;
		}
	    }
    }

    if (is_on(SHOW_TREE))
	printf("proposal from %s%s\n",crittab[bestprop].name,
	    (bestrating==pr[bestprop])?"":" and others");

    /* Negate the proposal */
    prop_copy(prop1,&p[bestprop]);
    prop_negate(leaf,prop2,&p[bestprop]);
}

#ifdef BOUND
crit_bprop(leaf,prop1)
Leaf leaf;
Prop *prop1;
{
register int i,j;
int k;
int lastprop;
int cnt;
int weight;
int bestprop, bestrating;
Prop p[N_CRITERIA];		/* Proposals from each criterion */
int pr[N_CRITERIA];		/* Ratings from proposer for p[i] */
int total;

    /* Have every real criterion do it's proposal */
    cnt = 0;
    for (i=0; i < N_CRITERIA-1; i++)
	if (weight=crittab[i].pweight)
	{
	    if ((pr[i] = weight*(*crittab[i].prop)(leaf,&p[i])) > 0)
	    {
		lastprop = i;
		cnt++;
	    }
	}
	else
		pr[i] = -1;

    if (cnt == 0)
    {
        /* If nothing proposed, use dflt criterion (always the last one) */
	if (is_on(SHOW_PROPS))
		printf("no proposals\n");
	bestprop = DEFAULT_CRITERION;
	bestrating = pr[bestprop] =
		(*crittab[bestprop].prop)(leaf,&p[bestprop]);
    }
    else if (cnt == 1)
    {
	/* If only one proposal, use it */
	if (is_on(SHOW_PROPS))
	    printf("only proposal is from %s weight %d\n",
		crittab[lastprop].name,pr[lastprop]);
	bestprop = lastprop;
	bestrating = pr[lastprop];
    }
    else
    {
    	/* Let criteria test each other's proposals, add em, pick highest */
	bestrating = -1;
	for (i=0; i <= lastprop; i++)
	    if (pr[i] >= 0)
	    {
		if (is_on(SHOW_PROPS))
		    printf("proposal from %s (%d)",crittab[i].name,pr[i]);

		/* Get other criteria's opinions, and sum them up */
		total = 0;
		for (j=0; j < N_CRITERIA; j++)
		{
		    if (j == i)
			total += pr[i];
		    /* Use only criteria that proposed things and DEFAULT */
		    else if (j == DEFAULT_CRITERION || pr[j] > 0)
		    {
			total += k = crittab[j].pweight *
			             (*crittab[j].test)(leaf,&p[i]);
			if (is_on(SHOW_PROPS) && k != 0)
			    printf(" + %s (%d)",crittab[j].name,k);
		    }
		}

		if (is_on(SHOW_PROPS))
		{
		    printf(" weight %d: ",total);
		    dump_prop(stdout,&p[i]);
		}

		/* Look for best total rating */
		if (abs(total) > abs(bestrating) ||
		    (bestrating == 0 && total == 0 &&
		    crittab[i].pweight > crittab[j].pweight))
		{
			bestrating = total;
			bestprop=i;
		}
	    }
    }

    if (is_on(SHOW_TREE))
	printf("proposal from %s%s\n",crittab[bestprop].name,
	    (bestrating==pr[bestprop])?"":" and others");

    /* Negate the proposal if it's negation is better */
    if (bestrating < 0)
	    prop_copy(prop1,&p[bestprop]);
    else
	    prop_negate(leaf,prop1,&p[bestprop]);
}
#endif BOUND

#endif TEST_PROPOSALS

dump_weights(fp)
FILE *fp;
{
int i;
    for (i=0; i < N_CRITERIA; i++)
	fprintf(fp,"%s weight = %d (evaluation), %d (proposing)\n",
	    crittab[i].name, crittab[i].eweight, crittab[i].pweight);
}

#ifdef SIZE
int sfc_leaf_size(); long sfc_stat_size();
int mani_leaf_size(); long mani_stat_size();
int mono_leaf_size(); long mono_stat_size();
int dflt_leaf_size(); long dflt_stat_size();

struct Crit_Size_List {
	int (*leaf)();	/* Pointer to leaf size routine */
	long (*stat)();	/* Pointer to static size routine */
	};

struct Crit_Size_List critsztab[N_CRITERIA] = {
	{ sfc_leaf_size, sfc_stat_size},
	{ mani_leaf_size, mani_stat_size},
	{ mono_leaf_size, mono_stat_size},
	{ dflt_leaf_size, dflt_stat_size}};

long crit_leaf_size(fp,leaf)
FILE *fp;
Leaf leaf;
{
long sz = 0;
long st;
int lt;
int i;
	/* print leaf sizes of all criteria */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
		{
			(*critsztab[i].leaf)(leaf,&st,&lt);
			fprintf(fp,"   %s = %ld/%d\n",crittab[i].name,st,lt);
			sz += st/lt;
		}
	return(sz);
}


long crit_stat_size(fp)
FILE *fp;
{
long sz = 0;
long st;
int i;
	/* print static sizes of all criteria */
	for (i=0; i < N_CRITERIA; i++)
		if (crittab[i].on)
		{
			sz += st = (*critsztab[i].stat)();
			fprintf(fp,"%s static size = %d\n",crittab[i].name,st);
		}
	return(sz);
}
#endif SIZE
