/* EXEC.H -- Executive Module Data structures */
/* (c) 1988 - University of Michigan */

#ifndef _H_EXEC
#define _H_EXEC

#ifndef _H_GLOBAL
#include "global.h"
#endif _H_GLOBAL

#ifndef _H_ASSERT
#include "assert.h"
#endif _H_ASSERT

#ifndef _H_LIST
#include "list.h"
#endif _H_LIST

#define DATA EXEC_MOD

#ifdef DISCARD
/* TREE_DATA
 * This is used to represent the search tree structure.  Only the proposals
 * that were used are saved.
 */
typedef struct Tree_Data Tnode;
struct Tree_Data {
	Tnode *papa;		/* pointer to parent tree node.
				   NULL if parent is root. */
	char type;		/* proposal used to make this from parent */
	union {
		SequProp s;
		TrajProp t;
		} store;	/* Storage for the proposal */
	};
#endif DISCARD

/* LEAF_DATA
 * This is the executive's leaf data.  It has the leaf's rating and a flag
 * which tells if the leaf represents a complete plan.  It also contains a
 * pointer to the next best leaf.  The prevopen and nextopen links actually
 * form a circular list, just to make updates easier.
 */

struct Leaf_Data {
	int id;			/* ID number of this leaf (for output only) */
	int rating;		/* This leafs optimistic rating */
	int completion;		/* How near completion is this?  0 = done */
	int depth;		/* How deep in the tree are we?  0 = root */
	struct Links l;		/* Links to other leaves */
#ifdef DISCARD
	Tnode *tree;		/* Pointer to parent in tree
				   NULL if this is the root node. */
	boolean collapsed;	/* Have we folded up this node? */
#endif DISCARD
	};

#ifdef DISCARD
extern Leaf orig;
#endif DISCARD


/* Declarations of local procedures */
Leaf copy_leaf(), find_pred(), clone_leaf();
Prop *get_tree_prop();

#endif _H_EXEC
