/* EXEC_DUMP.C -- Executive data structures dump module */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "global.h"
#include "config.h"
#include "set.h"
#include "assert.h"
#include "exec.h"

dump_leaf(leaf,fp)
Leaf leaf;
FILE *fp;
{
	exec_dump(leaf,fp);
#ifdef DISCARD
	if (!leaf[DATA]->collapsed)
#endif DISCARD
	{
		acg_dump(leaf,fp);
		crit_dump(leaf,fp);
	}
}

exec_dump(leaf,fp)
Leaf leaf;
FILE *fp;
{
#ifdef DISCARD
	fprintf(fp,"LEAF %d rating = %d completion = %d depth = %d",
		leaf[DATA]->id, leaf[DATA]->rating, leaf[DATA]->completion,
		leaf[DATA]->depth);
	fprintf(fp,leaf[DATA]->collapsed?" [COLLAPSED]\n":"\n");
#else
	fprintf(fp,"LEAF %d rating = %d completion = %d depth = %d\n",
		leaf[DATA]->id, leaf[DATA]->rating, leaf[DATA]->completion,
		leaf[DATA]->depth);
#endif DISCARD
}


#ifdef DISCARD
dump_ancestors(leaf,fp)
Leaf leaf;
FILE *fp;
{
Tnode *tree;
	for(tree=leaf[DATA]->tree; tree != NULL; tree = tree->papa)
		dump_prop(stdout,get_tree_prop(tree));
}
#endif DISCARD


/* DUMP_SEQUASRT(fp,sa)
 * print the sequencing assertion sa to fp.
 */

dump_sequasrt(fp,sa)
FILE *fp;
SequAsrt *sa;
{
	fprintf(fp,"{");
	pr_part_comma(fp,sa->p1)
	fprintf(fp,"} -> {");
	pr_part_comma(fp,sa->p2)
	fprintf(fp,"}\n");
}

/* DUMP_SEQUPROP(fp,sp)
 * print the sequencing proposal sp to fp.
 */

dump_sequprop(fp,sp)
FILE *fp;
SequProp *sp;
{
	fprintf(fp,"%s -> %s\n",part_name(sp->p1),part_name(sp->p2));
}

/* DUMP_TRAJASRT(fp,ta)
 * print the trajectory assertion/proposal ta to fp.
 */

dump_trajasrt(fp,ta)
FILE *fp;
TrajAsrt *ta;
{
	fprintf(fp,"%s uses ",part_name(ta->p));
	pr_traj_or(fp,&ta->t)
	fprintf(fp,"\n");
}

dump_prop(fp,pr)
FILE *fp;
Prop *pr;
{
	if (pr->type == SEQU)
		dump_sequprop(fp,pr->a.sequ);
	else
		dump_trajprop(fp,pr->a.traj);
}

#ifdef SIZE
long dump_leaf_size(fp,leaf)
FILE *fp;
Leaf leaf;
{
long st, sz,crit_leaf_size(),acg_leaf_size(),exec_leaf_size();

	sz = st = exec_leaf_size(leaf);
	fprintf(fp,"   EXEC = %ld\n",st);
#ifdef DISCARD
	if (!leaf[DATA]->collapsed)
#endif DISCARD
	{
		sz += st = acg_leaf_size(leaf);
		fprintf(fp,"   ACG  = %ld\n",st);
		sz += crit_leaf_size(fp,leaf);
	}
	fprintf(fp,"   TOTAL = %ld\n", sz);
	return(sz);
}
#endif SIZE
