/* EXEC_TOP -- top level executive routines */
/* (c) 1988 - University of Michigan */

#include <stdio.h>

#include "global.h"
#include "list.h"
#include "exec.h"
#include "assert.h"

#ifdef DISCARD
Leaf orig;		/* A clone of the original leaf */
int n_open;		/* Number of nodes currently open */
int col_count;		/* Number of collapses done */
int reb_count;		/* Number of rebuilds done */
#endif DISCARD
int exp_count;		/* Number of expansions done */

/* TOP_LEVEL
 * Solve the problem given in the problem file using the given criteria file.
 * Print the given number of solutions.
 */

top_level(prob_fp,crit_fp,debg_fp,solutions)
FILE *prob_fp, *crit_fp,*debg_fp;
int solutions;
{
#ifdef DISCARD
int open;
#endif DISCARD
Leaf best,tmp;
int scount;

	/* Initialization */
	exp_count = 0;
#ifdef DISCARD
	reb_count = col_count = 0;
#endif DISCARD
	initialize(prob_fp,crit_fp,debg_fp);
#ifdef DISCARD
	orig = clone_leaf(get_best());
	n_open = 1;
#endif DISCARD

	/* Loop for requested number of solutions */
#ifdef CRIPPLE
	while (get_best() != NULL)
#else
	for (scount=1; scount <= solutions && get_best() != NULL ; scount++)
#endif CRIPPLE
	{
	    /* Main Search Loop */
	    while(((best=get_best())!=NULL) && (best[DATA]->completion > 0))
	    {
#ifdef SIZE
		if (is_on(SHOW_SIZE))
		{
			dump_size(stdout);
			printf("Memory Usage = %ld\n",m_usage());
		}
#endif SIZE
#ifdef DISCARD
		if (best[DATA]->collapsed)
		{
		    if (is_on(SHOW_BOUND))
		    {
			    printf("\nREBUILDING ");
			    exec_dump(best,stdout);
		    }
		    rebuild_leaf(best);
		    /* don't add to open list -- will expand it immediately */
		    reb_count++;
		    n_open++;
		}
#endif DISCARD
		if (is_on(SHOW_LIST))
		{
			printf("\nLEAVES:\n");
			dump_tree(stdout);
			dump_open(stdout);
			putchar('\n');
		}
		if (is_on(SHOW_TREE) || is_on(SHOW_EXPAND))
		{
			printf("\nEXPANDING ");
			if (is_on(SHOW_EXPAND))
				dump_leaf(best,stdout);
			else
				exec_dump(best,stdout);
		}

		exp_count++;
		split_best();
#ifdef DISCARD
		while (n_open > 10 && m_usage() > MEMLIMIT)
		{
		    if ((tmp = rm_worst_open()) == NULL) break;

		    if (is_on(SHOW_BOUND))
		    {
			    printf("\nCOLLAPSING ");
			    exec_dump(tmp,stdout);
		    }
		    collapse_leaf(tmp);
		    col_count++;
		    n_open--;
		}
#endif DISCARD
	    }

#ifdef SIZE
	    if (is_on(SHOW_SIZE))
	    {
		    dump_size(stdout);
		    printf("Memory Usage = %ld\n",m_usage());
	    }
#endif SIZE
	    if ((tmp = rm_best())==NULL) break;
	    if (is_on(SHOW_SOLUTION))
	    {
		    /* Print the solution */
		    printf("\nSOLUTION %d (rating %d depth %d):\n",
			scount,tmp[DATA]->rating,tmp[DATA]->depth);
		    acg_print(tmp,stdout);
	    }

	    /* Drop the solved node */
	    drop_leaf(tmp);
#ifdef DISCARD
	    n_open--;
#endif DISCARD
	}

	if (is_on(SHOW_STATS))
	{
		printf("\n%d leaves expanded\n",exp_count);
#ifdef DISCARD
		printf("%d leaves collapsed\n",col_count);
		printf("%d leaves rebuilt\n",reb_count);
#endif DISCARD
		m_stat(stdout);
	}
}


/* SPLIT_BEST()
 * This splits the top plan on the list by splitting it in two and applying
 * new proposals to each.
 */

split_best()
{
register Leaf new, tmp, best;
Prop prop1,prop2;
Leaf preopen;

	/* Get a proposed assertion to branch on */
	best = get_best();
	crit_prop(best,&prop1,&prop2);

	/* Split the old leaf in two */
	new = copy_leaf(best);

	/* Apply the first proposal to the old leaf */
	leaf_switches(best[DATA]->id);
	if (is_on(SHOW_TREE)) dump_prop(stdout,&prop1);
#ifdef DISCARD
	save_prop(best,&prop1);
#endif DISCARD
	if (consequence(best,&prop1))
	{
		/* No Solution - Abandon the leaf */
		if (is_on(SHOW_TREE)) printf("CONFLICT--Dropping\n");
		tmp = rm_best();
		drop_leaf(tmp);
#ifdef DISCARD
		n_open--;
#endif DISCARD
	}
	else
	{
		/* Find the new rating and completion */
		best[DATA]->rating = crit_rate(best);
		best[DATA]->completion = acg_rate(best);
#ifdef CRIPPLE
		if (best[DATA]->completion > 0)
		      best[DATA]->rating = 99999;
#endif CRIPPLE

		if (is_on(SHOW_LEAVES))
			dump_leaf(best,stdout);
		else if (is_on(SHOW_TREE))
			exec_dump(best,stdout);

		change_best();
	}

	/* Apply the second proposal to the new leaf */
	leaf_switches(new[DATA]->id);
	if (is_on(SHOW_TREE)) dump_prop(stdout,&prop2);
#ifdef DISCARD
	save_prop(new,&prop2);
#endif DISCARD
	if (consequence(new,&prop2))
	{
		/* No Solution - Abandon the leaf */
		if (is_on(SHOW_TREE)) printf("CONFLICT--Dropping\n");
		drop_leaf(new);
		if (get_best() == NULL)
		{
			printf("Problem is not Solvable\n");
			return;
		}
	}
	else
	{
		/* Find the new rating and completion */
		new[DATA]->rating = crit_rate(new);
		new[DATA]->completion = acg_rate(new);

		if (is_on(SHOW_LEAVES))
			dump_leaf(new,stdout);
		else if (is_on(SHOW_TREE))
			exec_dump(new,stdout);

		add_open(new);
#ifdef DISCARD
		n_open++;
#endif DISCARD
	}
}


#ifdef DISCARD
rebuild_leaf(leaf)
Leaf leaf;
{
Tnode *tree;
Prop *prop;

	if (!leaf[DATA]->collapsed)
	{
		warning("Attempt to rebuild un-collapsed node ignored\n");
		return;
	}

	acg_copy(leaf,orig);
	crit_copy(leaf,orig);

	for(tree=leaf[DATA]->tree; tree != NULL; tree = tree->papa)
	{
	    prop = get_tree_prop(tree);
	    if (is_on(SHOW_BOUND) && is_on(SHOW_TREE))
		dump_prop(stdout,prop);
	    if (!reduce_prop(leaf,prop))
	    {
	        if (consequence(leaf,prop))
		    error("Conflict in rebuilding of leaf %d\n",leaf[DATA]->id);
	    }
	}
	leaf[DATA]->collapsed = FALSE;
}
#endif DISCARD

intr()
{
	if (is_on(SHOW_STATS))
	{
		printf("\n%d leaves expanded\n",exp_count);
#ifdef DISCARD
		printf("%d leaves collapsed\n",col_count);
		printf("%d leaves rebuilt\n",reb_count);
#endif DISCARD
		m_stat(stdout);
	}
	exit(0);
}
