/* GLOBAL.H -- Global types and declarations */
/* (c) 1988 - University of Michigan */

#ifndef _H_GLOBAL
#define _H_GLOBAL

#ifndef _H_VERSION
#include "version.h"
#endif _H_VERSION

typedef int boolean;

#undef TRUE
#undef FALSE
#define TRUE 1
#define FALSE 0

/* Module Numbers
 * Each module that operates on the search tree is assigned a unique number
 * with which to address it's data structures and identify it's error
 * messages.  Following those numbers are number for modules that don't
 * need to attach special data structures to tree leaves, but need to be
 * able to cough out error messages anyways.  These numbers should really
 * only be know to the modules themselves, but it is easier to maintain them
 * if we keep them all in one place.
 */

#define EXEC_MOD	0
#define ACG_MOD		1
#define MONO_MOD	2
#define MANI_MOD	3
#define SFC_MOD		4
#define N_TREE_MOD	5

#define CAD_MOD		N_TREE_MOD
#define CRIT_MOD	N_TREE_MOD+1
#define MSG_MOD		N_TREE_MOD+2
#define SET_MOD		N_TREE_MOD+3
#define TEST_MOD	N_TREE_MOD+4
#define DBUG_MOD	N_TREE_MOD+5
#define MEM_MOD		N_TREE_MOD+6
#define N_MOD		N_TREE_MOD+7

/* LEAF structure
 * This structure is the root for the data associated with a given leaf of
 * the search tree (only the leaves are saved).  It is a array of pointers
 * indexed by the module numbers which point to the modules data structure
 * for this leaf.  Notice that in different modules "struct Leaf_Data" is
 * defined differently, so anyone who tries to address a different module's
 * data is going to be in deep trouble.
 */

typedef struct Leaf_Data **Leaf;

/* Types for Part and Trajectory variables */

typedef int Part;
typedef int Traj;
typedef int Dire;

#ifndef _H_SET
#include "set.h"
#endif _H_SET

/* Declarations for cad_query routines */
char *traj_name(), *dire_name(), *part_name();
TrajSet *proposed_traj();
PartSet *constrained_parts();

/* Declarations for other fun routines */
boolean isdone();
TrajSet *possible_traj();
char *mm_alloc();
char *mm_init();

/* These are macros accessing global variables.  They could be changed to
 * real funciton calls, if actual object-orientedness were really vital,
 * rather than its mere appearance.  The externals are only directly accessed
 * in the cad module, where they should actually be.
 */

extern int n_parts, n_directions, n_trajectories;
#define part_number() n_parts
#define dire_number() n_directions
#define traj_number() n_trajectories

#define foreachpart(p) for (p=0;p<part_number();p++)
#define foreachtraj(t) for (t=0;t<traj_number();t++)

/* Declarations for unix routines */
char *malloc();

/* ERROR 
 * This is a macro to print an error message and exit. To make this work
 * nicely we have to do some ugly things with global variables and strange
 * handling of a function with variable numbers of arguments, but it looks
 * clean when you use it.
 */

extern int err_mod;
#define error err_mod=DATA,error_msg
#define warning err_mod=DATA,warn_msg
char *tof();

/* DEBUG SWITCHES
 * These are switches to do various diagnostic output.
 */

#define SHOW_PROPS        0
#define SHOW_RATINGS      1
#define SHOW_TREE         2
#define SHOW_LEAVES       3
#define SHOW_STATS        4
#define SHOW_WEIGHTS      5
#define SHOW_LIST         6
#define SHOW_ENTRIES      7
#define SHOW_EXPAND       8
#define SHOW_CONSTRAINTS  9
#define SHOW_SOLUTION	 10
#define SHOW_BOUND	 11
#define SHOW_SIZE	 12
#define N_SWITCHES       13
extern boolean debug_switch[N_SWITCHES];

#define turn_on(s) (debug_switch[s]=1)
#define turn_off(s) (debug_switch[s]=0)
#define is_on(s) debug_switch[s]

#endif _H_GLOBAL
