/* LIST.H -- LIST Module Data structures */
/* (c) 1988 - University of Michigan */

#ifndef _H_LIST
#define _H_LIST

#ifndef _H_GLOBAL
#include "global.h"
#endif _H_GLOBAL

/* Entry points */
void init_list();	/* initalize list with one leaf in it */
void add_open();	/* add a new open leaf to the list */
void change_best();	/* restructure list since best has changed */
Leaf get_best();	/* return pointer to best leaf - leaf it on list */
Leaf rm_best();		/* delete best leaf from list */
Leaf rm_worst_open();	/* worst open node is no longer open */

/* LEAF LINKS
 * This is the portion of the executive leaf data structure that contains
 * the links used in the list module.
 */

#ifdef HEAP

/* Two Heaps are kept.  The first one contains all leaves, with the best
 * rated on top.  The second contains all open leaves, with the worst on top.
 */

#define ALL 0		/* Identifier for heap of all nodes */
#ifdef DISCARD
#define OPEN 1		/* Identifier for heap of open nodes */
#define N_HEAP 2
#else
#define N_HEAP 1
#endif DISCARD

struct Links {
	short heap[N_HEAP];	/* index of corrisponding node in heap */
				/* -1 if removed from heap */
	};

#else

/* All nodes are on a linear sorted linked list.  Open nodes are on a circular
 * doubly linked list, also sorted
 */

struct Links { 
	Leaf next;              /* Pointer to next best leaf */
#ifdef DISCARD
	Leaf nextopen;          /* Pointer to next best uncollapsed leaf */
	Leaf prevopen;          /* Pointer to next worst uncollapsed leaf */
#endif DISCARD
	}; 

#endif HEAP

#endif _H_LIST
