/* (c) 1988 - University of Michigan */
#include <stdio.h>
#include "global.h"

/*#define DEBUGMEM /**/

#define DATA MEM_MOD

char *realloc();
char *malloc();

/*-----------------------------------------------------------------------*/
/* MM MEMORY MANAGER.  This is an extremely simple minded memory manager */
/* for handling one fixed size of memory block.  mm_init() returns a     */
/* pointer to a descriptor for a certain block size.  mm_alloc() and     */
/* mm_free can then be used to get and free such blocks.  mm_freesize    */
/* counts total wasted memory: memory that has been fetched from the     */
/* system, but not allocated, including the descriptor.			 */
/*-----------------------------------------------------------------------*/

struct mms {
	int	size;		/* Size of memory chunks allocated */
	int	block;		/* Number of chunks to allocate at once */
	struct mmhd *free;	/* First block on free list */
	};

struct mmhd {
	int	cnt;		/* Number of chunks in this block */
	struct mmhd *next;	/* Address of next block */
	};

char *mm_init(size,block)
int size;
int block;
{
struct mms *mm;

	/* Round it up to nearest even number */
	size = size + (size & 1);
	/* Make sure size is big enough to hold free list block header */
	if (sizeof(struct mmhd) > size) size = sizeof(struct mmhd);

	mm = (struct mms *)m_alloc(sizeof(struct mms));
	mm->size = size;
	mm->block = block;
	mm->free = NULL;
	return((char *)mm);
}

char *mm_alloc(mm)
struct mms *mm;
{
struct mmhd *hd1,*hd2;
char *p;

	if (mm->free == NULL)
	{
		/* malloc a new block */
		p = m_alloc(mm->size*mm->block);
		hd1 = mm->free = (struct mmhd *)(p + mm->size);
		hd1->cnt = mm->block - 1;
		hd1->next = NULL;
		return (p);
	}
	else
	{
		/* Use old memory */
		p = (char *)(hd1 = mm->free);
		if (hd1->cnt > 1)
		{
			hd2 = mm->free = (struct mmhd *)(p + mm->size);
			hd2->cnt = hd1->cnt - 1;
			hd2->next = hd1->next;
		}
		else
			mm->free = hd1->next;
		return (p);
	}
}

mm_free(mm,freed)
struct mms *mm;
struct mmhd *freed;
{
	freed->cnt = 1;
	freed->next = mm->free;
	mm->free = freed;
}

#ifdef SIZE
long mm_freesize(mm)
struct mms *mm;
{
long size;
struct mmhd *freeb;
	size = sizeof(struct mms);

	for (freeb = mm->free; freeb != NULL; freeb = freeb->next)
		size += freeb->cnt * mm->size;

	return(size);
}
#endif SIZE

/*----------------------------------------------------------------------*/

long mem_cnt = 0;
long mem_max = 0;

char *m_alloc(n)
int n;
{
#ifndef DEBUGMEM
	if ((mem_cnt += n) > mem_max) mem_max = mem_cnt;
	return(malloc(n));
#else
char *p;
	if ((mem_cnt += n) > mem_max) mem_max = mem_cnt;
	p = malloc(n);
	printf("%lx - Allocating %d new = %ld\n",p,n,mem_cnt);
	return(p);
#endif DEBUGMEM
}

m_free(p,n)
char *p;
int n;
{
	mem_cnt -= n;
#ifdef DEBUGMEM
	printf("%lx - Freeing %d new = %ld\n",p,n,mem_cnt);
#endif DEBUGMEM
	free(p);
}

char *m_realloc(p,o,n)
char *p;
int o,n;
{
#ifndef DEBUGMEM
	if ((mem_cnt += n - o) > mem_max) mem_max = mem_cnt;
	return(realloc(p,n));
#else
char *q;
	if ((mem_cnt += n - o) > mem_max) mem_max = mem_cnt;
	q = realloc(p,n);
	printf("%lx - Reallocating %d->%d new = %ld\n",p,o,n,mem_cnt);
	return(q);
#endif DEBUGMEM
}

m_stat(fp)
FILE *fp;
{
	fprintf(fp,"Max Memory usage = %ld\n",mem_max);
}

long m_usage()
{
	return(mem_cnt);
}
