/* SFC.H */
/* (c) 1988 - University of Michigan */

#ifndef _H_SFC
#define _H_SFC

/* Four possible attachment classes (split into five for convenience) */

#define AT 0    /* Attached */
#define L1 1    /* Lift-only */
#define L2 2    /* Lift-only on two rotational axis*/
#define RO 3    /* Rotate or lift */
#ifndef CONTACT
#define FR 4    /* Free */
#define N_TYPE 5
#else
#define CN 4    /* Surface Contact */
#define FR 5    /* Free */
#define N_TYPE 6
#endif CONTACT


/* Costs for the four (or five) classes of attachment */

extern int cost[N_TYPE];

/* Structures for Leaf_Data */

struct cpsl {
	PartSet s;              /* A partset */
	setsize n;              /* The cardinality of the set */
	setsize m;              /* Number of elements definately in the set */
	struct cpsl *next;      /* Next set in the list of sets */
	};

struct pfs {
	int type;               /* Which of the classes is this in? */
	PartSet at;             /* What set of parts would attach it */
	PartSet li;             /* What set of parts would lift from it */
	struct cpsl *ro;        /* What sets of parts would rotate on it */
	setsize nat;            /* How many parts in at? */
	setsize nli;            /* How many parts in li? */
	setsize nro;            /* How many sets in ro? */
	setsize mat;            /* How many parts in at are before the part? */
	setsize mli;            /* How many parts in li are before the part? */
	setsize mro;            /* How many sets in ro are before the part? */
#ifdef CONTACT
	PartSet cn;             /* What set of parts does it contact */
	setsize ncn;            /* How many parts in cn? */
	setsize mcn;            /* How many parts in cn are before the part? */
#endif CONTACT
	};

struct Leaf_Data {
#ifdef MERGE1
	int link_cnt;           /* How many leaves are using this */
#endif MERGE1
	struct pfs *pa;         /* Array of part attachment info */
				/* NULL if retired */
	setsize n[N_TYPE];      /* Number of parts in each class */
	int rating;		/* Rating for the plan. -1 if not computed */
	SequProp sprop;		/* Proposal for this node. */
	int sprate;		/* rating for prop. -2 if not computed */
	};

/*
 * This is a data structure used by the sfc_prop after being initialized by
 * sfc_init.  It is just an array of sequencing proposals with ratings, twice
 * the size of the number of parts.
 */

extern struct pl {
	SequProp sp;
#ifdef SFC_BMIN
	int min;
	int max;
#else
	int delta;
#endif SFC_BMIN
	} *plist;

extern int nprop;	/* Number of proposals on plist so far */

#endif _H_SFC
