/* SFC_RATE.C */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "config.h"
#include "set.h"
#include "global.h"
#include "assert.h"
#include "sfc.h"

#define DATA SFC_MOD

struct pl *plist;
int nprop;

/* SFC_TRAJ(leaf,ta)
 * SFC_SEQU(leaf,sa)
 * Update the data structure to account for the given assertions.
 */

sfc_traj(leaf,ta)
Leaf leaf;
TrajAsrt *ta;
{
	/* Do Nothing */
}

sfc_sequ(leaf,sa)
Leaf leaf;
SequAsrt *sa;
{
register struct pfs *pfa, *pfb;
register Part a,b;
struct cpsl *r,*t;
boolean changed = FALSE;

    if (is_on(SHOW_ENTRIES)) printf("sfc_sequ\n");
 
#ifdef RETIRE
    /* If retired, ignore updates */
    if (leaf[DATA]->pa == NULL) return;
#endif RETIRE

    foreach(a,sa->p1)
    {
	pfa = &leaf[DATA]->pa[a];
	foreach(b,sa->p2)
	{
	    /* Part b follows a, so knock it out of a's lists */

	    /* Attach list */
	    if (isin(&pfa->at,b))
	    {
#ifdef MERGE1
		if (leaf[DATA]->link_cnt > 1)
		{
			sfc_split(leaf);
			pfa = &leaf[DATA]->pa[a];
		}
#endif MERGE1
		changed = TRUE;
		remove1(&pfa->at,b);
		pfa->nat--;
	    }

	    /* Lift list */
	    if (isin(&pfa->li,b))
	    {
#ifdef MERGE1
		if (leaf[DATA]->link_cnt > 1)
		{
			sfc_split(leaf);
			pfa = &leaf[DATA]->pa[a];
		}
#endif MERGE1
		changed = TRUE;
		remove1(&pfa->li,b);
		pfa->nli--;
	    }

	    /* Rotation lists */
	    for (r=pfa->ro,t=NULL; r!=NULL; r=(t=r)->next)
	    {
		if (isin(&r->s,b))
		{
#ifdef MERGE1
		    if (leaf[DATA]->link_cnt > 1)
		    {
			    sfc_split(leaf);
			    pfa = &leaf[DATA]->pa[a];
			    for (r=pfa->ro,t=NULL; r!=NULL; r=(t=r)->next)
				if (isin(&r->s,b)) break;
		    }
#endif MERGE1
		    changed = TRUE;
		    remove1(&r->s,b);
		    /* if the set is empty, deallocate it */
		    if (--(r->n) == 0)
		    {
			if (t)
			    t->next = r->next;
			else
			    pfa->ro = r->next;
			m_free(r,sizeof(struct cpsl));
			pfa->nro--;
		    }
		}
	    }

#ifdef CONTACT
	    /* Contact list */
	    if (isin(&pfa->cn,b))
	    {
#ifdef MERGE1
		if (leaf[DATA]->link_cnt > 1)
		{
			sfc_split(leaf);
			pfa = &leaf[DATA]->pa[a];
		}
#endif MERGE1
		changed = TRUE;
		remove1(&pfa->cn,b);
		pfa->ncn--;
	    }
#endif CONTACT

	    /* Part a precedes b, so count it as surely being in lists */
	    pfb = &leaf[DATA]->pa[b];

	    /* Attach list */
	    if (isin(&pfb->at,a))
	    {
#ifdef MERGE1
		if (leaf[DATA]->link_cnt > 1)
		{
		    sfc_split(leaf);
		    pfa = &leaf[DATA]->pa[a];
		    pfb = &leaf[DATA]->pa[b];
		}
#endif MERGE1
		changed = TRUE;
		pfb->mat++;
	    }

	    /* Lift list */
	    if (isin(&pfb->li,a))
	    {
#ifdef MERGE1
		if (leaf[DATA]->link_cnt > 1)
		{
		    sfc_split(leaf);
		    pfa = &leaf[DATA]->pa[a];
		    pfb = &leaf[DATA]->pa[b];
		}
#endif MERGE1
		changed = TRUE;
		pfb->mli++;
	    }

	    /* Rotation lists */
	    for (r=pfb->ro; r!=NULL; r=r->next)
		if (isin(&r->s,a))
		{
#ifdef MERGE1
		    if (leaf[DATA]->link_cnt > 1)
		    {
			sfc_split(leaf);
			pfa = &leaf[DATA]->pa[a];
			pfb = &leaf[DATA]->pa[b];
			for (r=pfb->ro; r!=NULL; r=r->next)
			    if (isin(&r->s,a)) break;
		    }
#endif MERGE1
		    changed = TRUE;
		    if (r->m++ == 0) pfb->mro++;
		}

#ifdef CONTACT
	    /* Contact list */
	    if (isin(&pfb->cn,a))
	    {
#ifdef MERGE1
		if (leaf[DATA]->link_cnt > 1)
		{
		    sfc_split(leaf);
		    pfa = &leaf[DATA]->pa[a];
		    pfb = &leaf[DATA]->pa[b];
		}
#endif MERGE1
		changed = TRUE;
		pfb->mcn++;
	    }
#endif CONTACT
	}

	/* Reclassify part a and update type count */
	leaf[DATA]->n[pfa->type]--;
	leaf[DATA]->n[pfa->type = sfc_classify(pfa,AT)]++;
    }
    if (changed)
    {
	leaf[DATA]->sprate = -2;	/* Unknown */
	leaf[DATA]->rating = -1;	/* Unknown */
    }
}

#ifdef SFC2
int prop_adj;
#endif SFC2

/* SFC_RATE(leaf)
 * Make an optimistic estimate of the rating of this plan.
 */

int sfc_rate(leaf)
Leaf leaf;
{
register setsize *n = leaf[DATA]->n;
register int sum_cost=0;
register int i;

	if (is_on(SHOW_ENTRIES)) printf("sfc_rate\n");

	/* If we already have a rating, return that */
	if (leaf[DATA]->rating >= 0)
		return(leaf[DATA]->rating);

	n = leaf[DATA]->n;

	sum_cost=0;
	for (i=0; i<N_TYPE; i++)
		sum_cost += n[i]*cost[i];

	/* If there is no free part, free the freest rated one */
	if (n[FR] == 0)
	{
#ifdef CONTACT
		     if (n[CN] > 0) i = CN;
		else
#endif CONTACT
		     if (n[RO] > 0) i = RO;
		else if (n[L2] > 0) i = L2;
		else if (n[L1] > 0) i = L1;
		else i = AT;
		sum_cost = sum_cost - cost[i] + cost[FR];
	}

#ifdef SFC2
	/* Generate a proposal -- we get a tighter rating as a side effect */
	prop_adj = 0;
	sfc_work(leaf);
	sum_cost += prop_adj;
#endif SFC2

	leaf[DATA]->rating = sum_cost;
	return (sum_cost);
}

/* SFC_CLASSIFY(pf,start_type)
 * Determine what type the part is.  Start looking at the class start_type.
 */

int sfc_classify(pf,start_type)
struct pfs *pf;
{
	switch (start_type)
	{
	case AT:
		if (pf->nat > 0) return(AT);
	case L1:
		if (pf->nli > 0) return(L1);
	case L2:
		if (pf->nro > 1) return(L2);
	case RO:
		if (pf->nro > 0) return(RO);
#ifdef CONTACT
	case CN:
		if (pf->ncn > 0) return(CN);
#endif CONTACT
	case FR:
		return(FR);
	}
}


/* SFC_PROP(leaf,prop)
 * Set prop to a proposal that would be bad for sfc.  Return an estimate of how
 * much this proposal would hurt the rating things compared to it's negation.
 * If we find no proposals that would make an immediate difference, we look for
 * one that might lead us toward a place where we can make a difference and
 * return it with a rating of 0.  If we can't find that either, return -1.
 */

int sfc_prop(leaf,prop)
Leaf leaf;
Prop *prop;
{
#ifdef RETIRE
	/* If we are retired, just pass */
	if (leaf[DATA]->pa == NULL) return(-1);
#endif RETIRE

	if (is_on(SHOW_ENTRIES)) printf("sfc_prop\n");

	prop->type = SEQU;

	/* If we don't already have a proposal, find one */
	/* With SFC2, this should never happen */
	if (leaf[DATA]->sprate == -2)
		sfc_work(leaf);
	prop->a.sequ = &leaf[DATA]->sprop;
	return(leaf[DATA]->sprate);
}

sfc_work(leaf)
Leaf leaf;
{
Part p;
PartSet *ps;
struct pfs *pf=leaf[DATA]->pa;
int parts = part_number();
Part weak_prop = -1;
int i;
#ifdef SFC_BMIN
int best_min, best_max;
#endif SFC_BMIN
int best;
SequProp *sp;
#ifdef SFC_GUESS
struct cpsl *rsp;
#endif SFC_GUESS

	/* Build up a list of potential proposals and rate them */
	nprop = 0;
	for (p=0;p<parts;p++,pf++)
	{
	    switch (pf->type)
	    {
	    case AT:
		if (pf->mat > 0) break;
		if (pf->nat == 1)
		    add_plist(leaf, p, &pf->at, AT, TRUE);
		else
		    weak_prop = p;
		break;
	    case L1:
		if (pf->mli > 0) break;
		if (pf->nli == 1)
		    add_plist(leaf, p, &pf->li, L1, TRUE);
		else
		    weak_prop = p;
		break;
	    case L2:
		if (pf->mro > 1) break;
		if (pf->nro == 2)
		{
		    if (pf->ro->n == 1 && pf->ro->m < 1)
			add_plist(leaf, p, &pf->ro->s, L2, TRUE);
		    if (pf->ro->next->n == 1 && pf->ro->next->m < 1)
			add_plist(leaf, p, &pf->ro->next->s, L2, FALSE);
		}
		else
		    weak_prop = p;
		break;
	    case RO:
		if (pf->ro->m > 0) break;
		if (pf->ro->n == 1)
		    add_plist(leaf, p, &pf->ro->s, RO, TRUE);
		else
		    weak_prop = p;
		break;
#ifdef CONTACT
	    case CN:
		if (pf->mcn > 0) break;
		if (pf->ncn == 1)
		    add_plist(leaf, p, &pf->cn, CN, TRUE);
		else
		    weak_prop = p;
		break;
#endif CONTACT
	    }
	}

	if (nprop == 0)
	{
#ifdef RETIRE
		if (weak_prop == -1)
		{
			sfc_retire(leaf);
			return(-1);
		}
#endif RETIRE
#ifdef SFC_GUESS
		sp = &plist[0].sp;
		sp->p1 = weak_prop;
		pf = &leaf[DATA]->pa[weak_prop];
		switch(pf->type)
		{
		case AT:
			sp->p2 = first_in(&pf->at);
			break;
		case L1:
			sp->p2 = first_in(&pf->li);
			break;
		case L2:
			for (rsp=pf->ro; rsp != NULL; rsp=rsp->next)
			    if (rsp->m == 0)
			    {
				sp->p2 = first_in(&rsp->s);
				break;
			    }
			break;
		case RO:
			sp->p2 = first_in(&pf->ro->s);
			break;
#ifdef CONTACT
		case CN:
			sp->p2 = first_in(&pf->cn);
			break;
#endif CONTACT
		}
		leaf[DATA]->sprop.p1 = sp->p1;
		leaf[DATA]->sprop.p2 = sp->p2;
		return(leaf[DATA]->sprate = 0);
#else
		return(-1);
#endif SFC_GUESS
	}

	/* Look for best of those listed */
#ifdef SFC_BMIN
	best_min = -1;
	for (i=0; i<nprop; i++)
	    if (plist[i].min > best_min ||
		(plist[i].min == best_min && plist[i].max > best_max))
	    {
		sp = &plist[i].sp;
		best_min = plist[i].min;
		best_max = plist[i].max;
	    }
	best = best_min+best_max;
#else
	best = -1;
	for (i=0; i<nprop; i++)
	    if (plist[i].delta > best)
	    {
		sp = &plist[i].sp;
		best = plist[i].delta;
	    }
#endif SFC_BMIN

	leaf[DATA]->sprop.p1 = sp->p1;
	leaf[DATA]->sprop.p2 = sp->p2;
	return(leaf[DATA]->sprate = best);
}


/* How would the rating of part p change if we knocked it down from its old
 * rating?
 */

int find_delta(leaf,p,oldtype)
Leaf leaf;
Part p;
int oldtype;
{
int newtype;

	/* What difference would the proposal make? */
	newtype = sfc_classify(&leaf[DATA]->pa[p],oldtype+1);
	if (newtype == FR && leaf[DATA]->n[FR] == 0)
	{
		for (newtype--;newtype > oldtype;newtype--)
			if (leaf[DATA]->n[newtype] > 0)
				return(cost[newtype] - cost[oldtype]);
		return(0);
	}
	else
		return(cost[newtype] - cost[oldtype]);
}


/* Add the proposal that p preceeds some part in ps to the plist */

add_plist(leaf,p,ps,oldtype,countit)
Leaf leaf;
Part p;
PartSet *ps;
int oldtype;
boolean countit;
{
int newtype;
int i;
Part q = first_in(ps);
int delta;

    /* What difference would the proposal make? */
    delta = find_delta(leaf,p,oldtype);

    /* See if the negation is already on plist */
    if (q < p) /* Can't be on the list yet if q > p */
	for (i=0; i<nprop; i++)
	    if (plist[i].sp.p1 == q && plist[i].sp.p2 == p)
	    {
#ifdef SFC_BMIN
		if (plist[i].min > delta)
		{
			plist[i].max = plist[i].min; 
			plist[i].min = delta;
		}
		else
		{
			plist[i].max = delta; 
			/* Reverse the old proposal, since new one is worse */
			plist[i].sp.p1 = p;
			plist[i].sp.p2 = q;
		}
#ifdef SFC2
		if (countit) prop_adj += plist[i].min;
#endif SFC2
#else
#ifdef SFC_BSUM
		/* Reverse the proposal if new one is worse */
		if (plist[i].delta < delta)
		{
			plist[i].sp.p1 = p;
			plist[i].sp.p2 = q;
#ifdef SFC2
			if (countit) prop_adj += plist[i].delta;
#endif SFC2
		}
#ifdef SFC2
		else if (countit) prop_adj += delta;
#endif SFC2
		/* add this delta to the other delta */
		plist[i].delta += delta;
#else
		/* If so, subtract this delta from the other delta */
		/* Reverse the proposal if new one is worse */
		if (plist[i].delta < delta)
		{
#ifdef SFC2
			if (countit) prop_adj += plist[i].delta;
#endif SFC2
			plist[i].sp.p1 = p;
			plist[i].sp.p2 = q;
			plist[i].delta = delta - plist[i].delta;
		}
		else
		{
#ifdef SFC2
			if (countit) prop_adj += delta;
#endif SFC2
			plist[i].delta -= delta;
		}
#endif SFC_BSUM
#endif SFC_BMIN
		return;
	    }

    /* Otherwise, just add it to the list */
    plist[nprop].sp.p1 = p;
    plist[nprop].sp.p2 = q;
#ifdef SFC_BMIN
    plist[nprop].min = delta;
#else
    plist[nprop].delta = delta;
#endif SFC_BMIN
    nprop++;
}


#ifdef TEST_PROPOSALS

/* SFC_TEST(leaf,prop)
 * Return a (short sighted) rating for the given proposal as applied to the
 * given leaf.  The rating returned for the proposal generated by sfc_prop
 * should be the same as the rating sfc_prop gave for it.  The rating is
 * positive, if this proposal is bad for sfc, and negative if it is good.
 */

int sfc_test(leaf,prop)
Leaf leaf;
Prop *prop;
{
int delta1,delta2;
SequProp *sp;
struct pfs *pf=leaf[DATA]->pa;

	if (is_on(SHOW_ENTRIES))
	{
		printf("sfc_test ");
		dump_prop(stdout,prop);
		putchar('\n');
	}

#ifdef RETIRE
	/* If we are retired, no proposal matters */
	if (pf == NULL) return(0);
#endif RETIRE

	/* No Trajectory proposal can matter to us (short sightedly speaking) */
	if (prop->type != SEQU) return(0);

	sp = prop->a.sequ;

	/* Find the ratings of the proposal and its negation */
	delta1 = half_test(leaf,sp->p1,sp->p2);
	delta2 = half_test(leaf,sp->p2,sp->p1);

	/* Return the combined ratings */
#ifdef SFC_BMIN
	if (delta1 > delta2)
		return(delta1 + delta2);
	else
		return(- delta1 - delta2);
#else
#ifdef SFC_BSUM
	if (delta1 > delta2)
		return(delta1 + delta2);
	else
		return(- delta1 - delta2);
#else
	return(delta1 - delta2);
#endif SFC_BSUM
#endif SFC_BMIN
}


int half_test(leaf,p1,p2)
Leaf leaf;
Part p1,p2;
{
struct pfs *pf=leaf[DATA]->pa;
int delta;

    /* Find the rating change if p1->p2 is asserted */

    switch (pf[p1].type)
    {
    case AT:
	if (pf[p1].mat == 0 && isin(&pf[p1].at, p2))
		delta = find_delta(leaf,p1,AT);
	else
		delta = 0;
	break;
    case L1:
	if (pf[p1].mli == 0 && isin(&pf[p1].li, p2))
		delta = find_delta(leaf,p1,L1);
	else
		delta = 0;
	break;
    case L2:
	if (pf[p1].mro <= 1 && pf[p1].nro == 2)
	{
	    if (pf[p1].ro->m == 0 && isin(&pf[p1].ro->s, p2))
		delta = find_delta(leaf,p1,L2);
	    else if (pf[p1].ro->next->m == 0 && isin(&pf[p1].ro->next->s, p2))
		delta = find_delta(leaf,p1,L2);
	    else
		delta = 0;
	}
	else
	    delta = 0;
	break;
    case RO:
	if (pf[p1].ro->m == 0 && isin(&pf[p1].ro->s, p2))
		delta = find_delta(leaf,p1,RO);
	else
		delta = 0;
	break;
#ifdef CONTACT
    case CN:
	if (pf[p1].mcn == 0 && isin(&pf[p1].cn, p2))
		delta = find_delta(leaf,p1,CN);
	else
		delta = 0;
	break;
#endif CONTACT
    }
    return (delta);
}
#else
int sfc_test() {}
#endif TEST_PROPOSALS
