/* SFC_UTIL.C */
/* (c) 1988 - University of Michigan */

#include <stdio.h>
#include "config.h"
#include "set.h"
#include "global.h"
#include "assert.h"
#include "sfc.h"

#define DATA SFC_MOD

/* Costs for the four classes of attachment */

#ifndef CONTACT
int cost[N_TYPE] = {0,1,1,2,6};
#else
int cost[N_TYPE] = {0,1,1,2,4,6};
#endif CONTACT

/* SFC_INIT(leaf)
 * Initialize the data structure for the root leaf given, by allocating memory
 * and loading it up with the data from the cad interface.
 */

sfc_init(leaf)
Leaf leaf;
{
struct pfs *pf;
struct cpsl *rsp;
int parts = part_number();
Part p;
int i;
int n_axis;

	if (is_on(SHOW_ENTRIES)) printf("sfc_init\n");

	/* Set up our local static data structures */
	plist = (struct pl *) m_alloc(sizeof(struct pl)*2*parts);

	/* Allocate memory for the leaf */
	leaf[DATA] = (struct Leaf_Data *) m_alloc(sizeof(struct Leaf_Data));
	pf = leaf[DATA]->pa = (struct pfs *) m_alloc(sizeof(struct pfs)*parts);

#ifdef MERGE1
	leaf[DATA]->link_cnt = 1;
#endif MERGE1
	leaf[DATA]->sprate = -2;	/* Mark as unknown */
	leaf[DATA]->rating = -1;	/* Mark as unknown */

	/* Initialize type counts */
	for (i=0;i<N_TYPE;i++)
		leaf[DATA]->n[i] = 0;

	/* Get the data from the CAD interface, and set the cardinalities */
	for (p = 0; p < parts; p++,pf++)
	{
		setcopy(&pf->at,get_attached(p));
		pf->nat = cardinality(&pf->at);
		pf->mat = 0;

		setcopy(&pf->li,get_lift(p));
		pf->nli = cardinality(&pf->li);
		pf->mli = 0;

		pf->ro = NULL;
		pf->nro = 0;
		pf->mro = 0;
		for (n_axis = get_n_axis(p)-1; n_axis >= 0; n_axis--)
		{
			rsp = (struct cpsl *) m_alloc(sizeof(struct cpsl));
			rsp->next = pf->ro;
			pf->ro = rsp;
			pf->nro++;
			setcopy(&rsp->s,get_rotate(p,n_axis));
			rsp->n = cardinality(&rsp->s);
			rsp->m = 0;
		}
#ifdef CONTACT
		setcopy(&pf->cn,get_contact(p));
		pf->ncn = cardinality(&pf->cn);
		pf->mcn = 0;
#endif CONTACT
		leaf[DATA]->n[pf->type = sfc_classify(pf,AT)]++;
	}
}


/* SFC_COPY(new,old)
 * Copy the given leaf into a new one.  This is done by making both point to
 * the same leaf if MERGE1 is defined.  Otherwise a physical copy is always
 * made.
 *
 * SFC_SPLIT(leaf)
 * This takes a leaf with a link count greater than one and divides it into a
 * separate leaf with it's own memory.  Used if MERGE1 is defined only.
 */

#ifdef MERGE1
sfc_copy(new,old)
Leaf new,old;
{
	if (is_on(SHOW_ENTRIES)) printf("sfc_copy\n");

	new[DATA] = old[DATA];
	new[DATA]->link_cnt++;
}

sfc_split(new)
Leaf new;
{
struct Leaf_Data *od = new[DATA];
#else
sfc_copy(new,old)
Leaf new,old;
{
struct Leaf_Data *od = old[DATA];
#endif
struct pfs *npf,*opf=od->pa;
struct cpsl *nrsp, *orsp;
int parts = part_number();
int i;
Part p;

	if (is_on(SHOW_ENTRIES)) printf("sfc_split\n");

	/* Allocate memory for the new leaf */
	new[DATA] = (struct Leaf_Data *) m_alloc(sizeof(struct Leaf_Data));

	/* Copy type counts */
	for (i=0;i<N_TYPE;i++)
		new[DATA]->n[i] = od->n[i];

#ifdef MERGE1
	new[DATA]->link_cnt = 1;
	od->link_cnt--;
#endif MERGE1

#ifdef RETIRE
	if (od->pa == NULL)
	    new[DATA]->pa = NULL;
	else
	{
#endif RETIRE
	    npf = new[DATA]->pa =
		    (struct pfs *) m_alloc(sizeof(struct pfs)*parts);

	    /* Copy part array of part sets */
	    for (p = 0; p < parts; p++,opf++,npf++)
	    {
		npf->type = opf->type;

		setcopy(&npf->at,&opf->at);
		npf->nat = opf->nat;
		npf->mat = opf->mat;

		setcopy(&npf->li,&opf->li);
		npf->nli = opf->nli;
		npf->mli = opf->mli;

		npf->nro = opf->nro;
		npf->mro = opf->mro;
		npf->ro = NULL;
		for (orsp = opf->ro; orsp != NULL; orsp=orsp->next)
		{
		    nrsp = (struct cpsl *) m_alloc(sizeof(struct cpsl));
		    nrsp->next = npf->ro;
		    npf->ro = nrsp;
		    setcopy(&nrsp->s,&orsp->s);
		    nrsp->n = orsp->n;
		    nrsp->m = orsp->m;
		}
#ifdef CONTACT
		setcopy(&npf->cn,&opf->cn);
		npf->ncn = opf->ncn;
		npf->mcn = opf->mcn;
#endif CONTACT
	    }
#ifdef RETIRE
	}
#endif RETIRE
} /* } */


/* SFC_DROP(leaf)
 * Release all memory used by our data structure in the given leaf.
 */

sfc_drop(leaf)
Leaf leaf;
{
Part p;
struct pfs *pf=leaf[DATA]->pa;
int parts = part_number();
struct cpsl *rsp,*nrsp;

	if (is_on(SHOW_ENTRIES)) printf("sfc_drop\n");

#ifdef MERGE1
	if (--leaf[DATA]->link_cnt == 0)
#endif MERGE1
	{
		sfc_retire(leaf);
		m_free(leaf[DATA],sizeof(struct Leaf_Data));
	}
	leaf[DATA] = NULL;
}

sfc_retire(leaf)
Leaf leaf;
{
Part p;
int parts = part_number();
struct pfs *pf=leaf[DATA]->pa;
struct cpsl *rsp,*nrsp;

	if (is_on(SHOW_ENTRIES)) printf("sfc_retire\n");

	if (pf == NULL) return;
	for (p=0; p<parts; p++)
	{
		for(rsp=pf[p].ro; rsp!=NULL; rsp=nrsp)
		{
			nrsp = rsp->next;
			m_free(rsp,sizeof(struct cpsl));
		}
	}
	m_free(pf,sizeof(struct pfs)*parts);
	leaf[DATA]->pa = NULL;
}

/* SFC_DUMP(leaf,fp)
 * print the contents of the our leaf data on the given device.
 */

sfc_dump(leaf,fp)
Leaf leaf;
FILE *fp;
{
Part p;
int parts = part_number();
struct pfs *pf=leaf[DATA]->pa;
struct cpsl *rsp;
int i;

#ifdef RETIRE
	if (pf == NULL)
#ifdef MERGE1
	    fprintf(fp,"SFC Retired. (lc=%d)\n",leaf[DATA]->link_cnt);
#else
	    fprintf(fp,"SFC Retired.\n");
#endif MERGE1
        else
#endif RETIRE
	{
#ifdef MERGE1
	    fprintf(fp,"SFC. (lc=%d)\n",leaf[DATA]->link_cnt);
#else
	    fprintf(fp,"SFC.\n");
#endif MERGE1
	    for (p=0; p<parts;p++)
	    {
		fprintf(fp,"  Part %s:\n",part_name(p));
		fprintf(fp,"     attaches to (%d/%d) ",pf[p].mat,pf[p].nat);
		pr_part_or(fp,&pf[p].at);
		fprintf(fp,"\n     lifts from (%d/%d) ",pf[p].mli,pf[p].nli);
		pr_part_or(fp,&pf[p].li);
		for (rsp=pf[p].ro,i=1; rsp != NULL; rsp=rsp->next,i++)
		{
			fprintf(fp,"\n     rotates about %d on (%d/%d) ",
				i,rsp->m,rsp->n);
			pr_part_or(fp,&rsp->s);
		}
#ifdef CONTACT
		fprintf(fp,"\n     contacts (%d/%d) ",pf[p].mcn,pf[p].ncn);
		pr_part_or(fp,&pf[p].cn);
#endif CONTACT
		fprintf(fp,"\n     type %d\n",pf[p].type);
	    }
	}
	fprintf(fp,"  type counts: ");
	for (i=0;i<N_TYPE;i++)
		fprintf(fp,"%d ",leaf[DATA]->n[i]);
	fprintf(fp,"\n");
}

#ifdef SIZE
/* SFC_SIZE
 * Return size of the SFC data structure for the given leaf.
 */

sfc_leaf_size(leaf,size,links)
Leaf leaf;
long *size;
int *links;
{
struct pfs *pf=leaf[DATA]->pa;
int parts = part_number();
struct cpsl *rsp;
Part p;

#ifdef MERGE1
	*links = leaf[DATA]->link_cnt;
#else
	*links = 1;
#endif MERGE1

	*size = sizeof(struct Leaf_Data);

#ifdef RETIRE
	if (pf == NULL) return;
#endif RETIRE

	*size += sizeof(struct pfs)*parts;

	for (p=0; p<parts;p++)
		for(rsp=pf[p].ro; rsp!=NULL; rsp=rsp->next)
			*size += sizeof(struct cpsl);
}

long sfc_stat_size()
{
	return((long)(sizeof(struct pl)*2*part_number()));
}
#endif SIZE
